package sun.java2d.opengl;

import sun.java2d.DisposerRecord;
import sun.java2d.pipe.BufferedContext;
import sun.java2d.pipe.hw.AccelDeviceEventNotifier;
import sun.java2d.pipe.hw.AccelDeviceEventListener;
import sun.java2d.Surface;
import sun.java2d.pipe.hw.AccelSurface;
import sun.java2d.pipe.hw.AccelTypedVolatileImage;
import java.awt.Graphics;
import sun.java2d.SunGraphics2D;
import java.awt.Color;
import sun.awt.image.SurfaceManager;
import sun.awt.image.SunVolatileImage;
import java.awt.image.VolatileImage;
import java.awt.AWTException;
import java.awt.Component;
import sun.java2d.windows.GDIWindowSurfaceData;
import sun.awt.windows.WComponentPeer;
import java.awt.color.ColorSpace;
import java.awt.image.DirectColorModel;
import java.awt.image.ColorModel;
import sun.java2d.SurfaceData;
import sun.java2d.Disposer;
import sun.awt.Win32GraphicsDevice;
import sun.java2d.pipe.hw.ContextCapabilities;
import java.awt.BufferCapabilities;
import java.awt.ImageCapabilities;
import sun.awt.Win32GraphicsConfig;

public class WGLGraphicsConfig extends Win32GraphicsConfig implements OGLGraphicsConfig
{
    protected static boolean wglAvailable;
    private static ImageCapabilities imageCaps;
    private BufferCapabilities bufferCaps;
    private long pConfigInfo;
    private ContextCapabilities oglCaps;
    private OGLContext context;
    private Object disposerReferent;
    
    public static native int getDefaultPixFmt(int p0);
    
    private static native boolean initWGL();
    
    private static native long getWGLConfigInfo(int p0, int p1);
    
    private static native int getOGLCapabilities(long p0);
    
    protected WGLGraphicsConfig(Win32GraphicsDevice win32GraphicsDevice, int n, long pConfigInfo, ContextCapabilities oglCaps) {
        super(win32GraphicsDevice, n);
        this.disposerReferent = new Object();
        this.pConfigInfo = pConfigInfo;
        this.oglCaps = oglCaps;
        this.context = new OGLContext(OGLRenderQueue.getInstance(), this);
        Disposer.addRecord(this.disposerReferent, new WGLGraphicsConfig.WGLGCDisposerRecord(this.pConfigInfo, win32GraphicsDevice.getScreen()));
    }
    
    public Object getProxyKey() {
        return this;
    }
    
    public SurfaceData createManagedSurface(int n, int n2, int n3) {
        return WGLSurfaceData.createData(this, n, n2, this.getColorModel(n3), null, 3);
    }
    
    public static WGLGraphicsConfig getConfig(Win32GraphicsDevice win32GraphicsDevice, int n) {
        if (!WGLGraphicsConfig.wglAvailable) {
            return null;
        }
        long configInfo = 0L;
        final String[] array = { null };
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            OGLContext.invalidateCurrentContext();
            WGLGraphicsConfig.WGLGetConfigInfo wglGetConfigInfo = new WGLGetConfigInfo(win32GraphicsDevice.getScreen(), n);
            instance.flushAndInvokeNow(wglGetConfigInfo);
            configInfo = wglGetConfigInfo.getConfigInfo();
            if (configInfo != 0L) {
                OGLContext.setScratchSurface(configInfo);
                instance.flushAndInvokeNow(new Runnable() {
                    @Override
                    public void run() {
                        array[0] = OGLContext.getOGLIdString();
                    }
                });
            }
        }
        finally {
            instance.unlock();
        }
        if (configInfo == 0L) {
            return null;
        }
        return new WGLGraphicsConfig(win32GraphicsDevice, n, configInfo, new OGLContext.OGLContextCaps(getOGLCapabilities(configInfo), array[0]));
    }
    
    public static boolean isWGLAvailable() {
        return WGLGraphicsConfig.wglAvailable;
    }
    
    public final boolean isCapPresent(int n) {
        return (this.oglCaps.getCaps() & n) != 0x0;
    }
    
    public final long getNativeConfigInfo() {
        return this.pConfigInfo;
    }
    
    public final OGLContext getContext() {
        return this.context;
    }
    
    public synchronized void displayChanged() {
        super.displayChanged();
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            OGLContext.invalidateCurrentContext();
        }
        finally {
            instance.unlock();
        }
    }
    
    public ColorModel getColorModel(int n) {
        switch (n) {
            case 1: {
                return new DirectColorModel(24, 16711680, 65280, 255);
            }
            case 2: {
                return new DirectColorModel(25, 16711680, 65280, 255, 16777216);
            }
            case 3: {
                return new DirectColorModel(ColorSpace.getInstance(1000), 32, 16711680, 65280, 255, -16777216, true, 3);
            }
            default: {
                return null;
            }
        }
    }
    
    public String toString() {
        return "WGLGraphicsConfig[dev=" + this.screen + ",pixfmt=" + this.visual + "]";
    }
    
    public SurfaceData createSurfaceData(WComponentPeer wComponentPeer, int n) {
        WGLSurfaceData.WGLWindowSurfaceData wglWindowSurfaceData = WGLSurfaceData.createData(wComponentPeer);
        if (wglWindowSurfaceData == null) {
            wglWindowSurfaceData = GDIWindowSurfaceData.createData(wComponentPeer);
            return wglWindowSurfaceData;
        }
        return wglWindowSurfaceData;
    }
    
    public void assertOperationSupported(Component component, int n, BufferCapabilities bufferCapabilities) throws AWTException {
        if (n > 2) {
            throw new AWTException("Only double or single buffering is supported");
        }
        if (!this.getBufferCapabilities().isPageFlipping()) {
            throw new AWTException("Page flipping is not supported");
        }
        if (bufferCapabilities.getFlipContents() == BufferCapabilities.FlipContents.PRIOR) {
            throw new AWTException("FlipContents.PRIOR is not supported");
        }
        return;
    }
    
    public VolatileImage createBackBuffer(WComponentPeer wComponentPeer) {
        Component comp = (Component)wComponentPeer.getTarget();
        return new SunVolatileImage(comp, Math.max(1, comp.getWidth()), Math.max(1, comp.getHeight()), Boolean.TRUE);
    }
    
    public void flip(WComponentPeer wComponentPeer, Component component, VolatileImage img, int n, int n2, int n3, int n4, BufferCapabilities.FlipContents flipContents) {
        if (flipContents == BufferCapabilities.FlipContents.COPIED) {
            SurfaceData primarySurfaceData = SurfaceManager.getManager(img).getPrimarySurfaceData();
            if ((primarySurfaceData instanceof WGLSurfaceData.WGLVSyncOffScreenSurfaceData)) {
                SunGraphics2D sunGraphics2D = new SunGraphics2D(((WGLSurfaceData.WGLVSyncOffScreenSurfaceData)primarySurfaceData).getFlipSurface(), Color.black, Color.white, null);
                try {
                    sunGraphics2D.drawImage(img, 0, 0, null);
                }
                finally {
                    sunGraphics2D.dispose();
                }
            }
            else {
                Graphics graphics = wComponentPeer.getGraphics();
                try {
                    graphics.drawImage(img, n, n2, n3, n4, n, n2, n3, n4, null);
                }
                finally {
                    graphics.dispose();
                }
                return;
            }
        }
        else if (flipContents == BufferCapabilities.FlipContents.PRIOR) {
            return;
        }
        OGLSurfaceData.swapBuffers(wComponentPeer.getData());
        if (flipContents == BufferCapabilities.FlipContents.BACKGROUND) {
            Graphics graphics2 = img.getGraphics();
            try {
                graphics2.setColor(component.getBackground());
                graphics2.fillRect(0, 0, img.getWidth(), img.getHeight());
            }
            finally {
                graphics2.dispose();
            }
        }
    }
    
    public BufferCapabilities getBufferCapabilities() {
        if (this.bufferCaps == null) {
            this.bufferCaps = new WGLGraphicsConfig.WGLBufferCaps(this.isCapPresent(65536));
        }
        return this.bufferCaps;
    }
    
    public ImageCapabilities getImageCapabilities() {
        return WGLGraphicsConfig.imageCaps;
    }
    
    public VolatileImage createCompatibleVolatileImage(int width, int height, int transparency, int accType) {
        if ((accType != 5 && accType != 3) || transparency == 2 || (accType == 5 && !this.isCapPresent(12))) {
            return null;
        }
        AccelTypedVolatileImage accelTypedVolatileImage = new AccelTypedVolatileImage(this, width, height, transparency, accType);
        Surface destSurface = accelTypedVolatileImage.getDestSurface();
        if (!(destSurface instanceof AccelSurface) || ((AccelSurface)destSurface).getType() != accType) {
            accelTypedVolatileImage.flush();
            accelTypedVolatileImage = null;
            return accelTypedVolatileImage;
        }
        return accelTypedVolatileImage;
    }
    
    public ContextCapabilities getContextCapabilities() {
        return this.oglCaps;
    }
    
    public void addDeviceEventListener(AccelDeviceEventListener accelDeviceEventListener) {
        AccelDeviceEventNotifier.addListener(accelDeviceEventListener, this.screen.getScreen());
    }
    
    public void removeDeviceEventListener(AccelDeviceEventListener accelDeviceEventListener) {
        AccelDeviceEventNotifier.removeListener(accelDeviceEventListener);
    }
    
    static long access$200(int n, int n2) {
        return getWGLConfigInfo(n, n2);
    }
    
    static {
        WGLGraphicsConfig.imageCaps = new WGLGraphicsConfig.WGLImageCaps();
        WGLGraphicsConfig.wglAvailable = initWGL();
    }
    
    private static class WGLGetConfigInfo implements Runnable
    {
        private int screen;
        private int pixfmt;
        private long cfginfo;
        
        private WGLGetConfigInfo(int screen, int pixfmt) {
            this.screen = screen;
            this.pixfmt = pixfmt;
        }
        
        @Override
        public void run() {
            this.cfginfo = WGLGraphicsConfig.access$200(this.screen, this.pixfmt);
        }
        
        public long getConfigInfo() {
            return this.cfginfo;
        }
    }
    
    private static class WGLGCDisposerRecord implements DisposerRecord
    {
        private long pCfgInfo;
        private int screen;
        
        public WGLGCDisposerRecord(long pCfgInfo, int n) {
            this.pCfgInfo = pCfgInfo;
        }
        
        @Override
        public void dispose() {
            OGLRenderQueue instance = OGLRenderQueue.getInstance();
            instance.lock();
            try {
                instance.flushAndInvokeNow(new Runnable() {
                    @Override
                    public void run() {
                        AccelDeviceEventNotifier.eventOccured(WGLGraphicsConfig.WGLGCDisposerRecord.this.screen, 0);
                        AccelDeviceEventNotifier.eventOccured(WGLGraphicsConfig.WGLGCDisposerRecord.this.screen, 1);
                    }
                });
            }
            finally {
                instance.unlock();
            }
            if (this.pCfgInfo != 0L) {
                OGLRenderQueue.disposeGraphicsConfig(this.pCfgInfo);
                this.pCfgInfo = 0L;
            }
        }
    }
    
    private static class WGLBufferCaps extends BufferCapabilities
    {
        public WGLBufferCaps(boolean b) {
            super(WGLGraphicsConfig.imageCaps, WGLGraphicsConfig.imageCaps, b ? BufferCapabilities.FlipContents.UNDEFINED : null);
        }
    }
    
    private static class WGLImageCaps extends ImageCapabilities
    {
        private WGLImageCaps() {
            super(true);
        }
        
        @Override
        public boolean isTrueVolatile() {
            return true;
        }
    }
}
